@@ -6,10 +6,14 @@
 
 """Extractors for https://www.facebook.com/"""
 
-from .common import Extractor, Message
-from .. import text, exception
+from .common import Extractor, Message, Dispatch
+from .. import text, util, exception
+from ..cache import memcache
 
 BASE_PATTERN = r"(?:https?://)?(?:[\w-]+\.)?facebook\.com"
+USER_PATTERN = (BASE_PATTERN +
+                r"/(?!media/|photo/|photo.php|watch/)"
+                r"(?:profile\.php\?id=|people/[^/?#]+/)?([^/?&#]+)")
 
 
 class FacebookExtractor(Extractor):
@@ -20,9 +24,6 @@ class FacebookExtractor(Extractor):
     filename_fmt = "{id}.{extension}"
     archive_fmt = "{id}.{extension}"
 
-    set_url_fmt = root + "/media/set/?set={set_id}"
-    photo_url_fmt = root + "/photo/?fbid={photo_id}&set={set_id}"
-
     def _init(self):
         headers = self.session.headers
         headers["Accept"] = (
@@ -37,21 +38,20 @@ class FacebookExtractor(Extractor):
         self.videos = self.config("videos", True)
         self.author_followups = self.config("author-followups", False)
 
-    @staticmethod
-    def decode_all(txt):
+    def decode_all(self, txt):
         return text.unescape(
-            txt.encode("utf-8").decode("unicode_escape")
+            txt.encode().decode("unicode_escape")
+            .encode("utf_16", "surrogatepass").decode("utf_16")
         ).replace("\\/", "/")
 
-    @staticmethod
-    def parse_set_page(set_page):
+    def parse_set_page(self, set_page):
         directory = {
             "set_id": text.extr(
                 set_page, '"mediaSetToken":"', '"'
             ) or text.extr(
                 set_page, '"mediasetToken":"', '"'
             ),
-            "username": FacebookExtractor.decode_all(
+            "username": self.decode_all(
                 text.extr(
                     set_page, '"user":{"__isProfile":"User","name":"', '","'
                 ) or text.extr(
@@ -61,7 +61,8 @@ class FacebookExtractor(Extractor):
             "user_id": text.extr(
                 set_page, '"owner":{"__typename":"User","id":"', '"'
             ),
-            "title": FacebookExtractor.decode_all(text.extr(
+            "user_pfbid": "",
+            "title": self.decode_all(text.extr(
                 set_page, '"title":{"text":"', '"'
             )),
             "first_photo_id": text.extr(
@@ -74,10 +75,18 @@ class FacebookExtractor(Extractor):
             )
         }
 
+        if directory["user_id"].startswith("pfbid"):
+            directory["user_pfbid"] = directory["user_id"]
+            directory["user_id"] = (
+                text.extr(
+                    set_page, '"actors":[{"__typename":"User","id":"', '"') or
+                text.extr(
+                    set_page, '"userID":"', '"') or
+                directory["set_id"].split(".")[1])
+
         return directory
 
-    @staticmethod
-    def parse_photo_page(photo_page):
+    def parse_photo_page(self, photo_page):
         photo = {
             "id": text.extr(
                 photo_page, '"__isNode":"Photo","id":"', '"'
@@ -87,21 +96,23 @@ class FacebookExtractor(Extractor):
                 '"url":"https:\\/\\/www.facebook.com\\/photo\\/?fbid=',
                 '"'
             ).rsplit("&set=", 1)[-1],
-            "username": FacebookExtractor.decode_all(text.extr(
+            "username": self.decode_all(text.extr(
                 photo_page, '"owner":{"__typename":"User","name":"', '"'
             )),
             "user_id": text.extr(
                 photo_page, '"owner":{"__typename":"User","id":"', '"'
             ),
-            "caption": FacebookExtractor.decode_all(text.extr(
+            "user_pfbid": "",
+            "caption": self.decode_all(text.extr(
                 photo_page,
                 '"message":{"delight_ranges"',
                 '"},"message_preferred_body"'
             ).rsplit('],"text":"', 1)[-1]),
-            "date": text.parse_timestamp(text.extr(
-                photo_page, '\\"publish_time\\":', ','
-            )),
-            "url": FacebookExtractor.decode_all(text.extr(
+            "date": text.parse_timestamp(
+                text.extr(photo_page, '\\"publish_time\\":', ',') or
+                text.extr(photo_page, '"created_time":', ',')
+            ),
+            "url": self.decode_all(text.extr(
                 photo_page, ',"image":{"uri":"', '","'
             )),
             "next_photo_id": text.extr(
@@ -115,6 +126,11 @@ class FacebookExtractor(Extractor):
             )
         }
 
+        if photo["user_id"].startswith("pfbid"):
+            photo["user_pfbid"] = photo["user_id"]
+            photo["user_id"] = text.extr(
+                photo_page, r'\"content_owner_id_new\":\"', r'\"')
+
         text.nameext_from_url(photo["url"], photo)
 
         photo["followups_ids"] = []
@@ -131,8 +147,7 @@ class FacebookExtractor(Extractor):
 
         return photo
 
-    @staticmethod
-    def parse_post_page(post_page):
+    def parse_post_page(self, post_page):
         first_photo_url = text.extr(
             text.extr(
                 post_page, '"__isMedia":"Photo"', '"target_group"'
@@ -146,13 +161,12 @@ class FacebookExtractor(Extractor):
 
         return post
 
-    @staticmethod
-    def parse_video_page(video_page):
+    def parse_video_page(self, video_page):
         video = {
             "id": text.extr(
                 video_page, '\\"video_id\\":\\"', '\\"'
             ),
-            "username": FacebookExtractor.decode_all(text.extr(
+            "username": self.decode_all(text.extr(
                 video_page, '"actors":[{"__typename":"User","name":"', '","'
             )),
             "user_id": text.extr(
@@ -165,7 +179,7 @@ class FacebookExtractor(Extractor):
         }
 
         if not video["username"]:
-            video["username"] = FacebookExtractor.decode_all(text.extr(
+            video["username"] = self.decode_all(text.extr(
                 video_page,
                 '"__typename":"User","id":"' + video["user_id"] + '","name":"',
                 '","'
@@ -177,7 +191,7 @@ class FacebookExtractor(Extractor):
 
         audio = {
             **video,
-            "url": FacebookExtractor.decode_all(text.extr(
+            "url": self.decode_all(text.extr(
                 text.extr(
                     first_video_raw,
                     "AudioChannelConfiguration",
@@ -194,7 +208,7 @@ class FacebookExtractor(Extractor):
             first_video_raw, 'FBQualityLabel=\\"', '\\u003C\\/BaseURL>'
         ):
             resolution = raw_url.split('\\"', 1)[0]
-            video["urls"][resolution] = FacebookExtractor.decode_all(
+            video["urls"][resolution] = self.decode_all(
                 raw_url.split('BaseURL>', 1)[1]
             )
 
@@ -222,36 +236,33 @@ class FacebookExtractor(Extractor):
         res = self.request(url, **kwargs)
 
         if res.url.startswith(self.root + "/login"):
-            raise exception.AuthenticationError(
-                "You must be logged in to continue viewing images." +
-                LEFT_OFF_TXT
+            raise exception.AuthRequired(
+                message=(f"You must be logged in to continue viewing images."
+                         f"{LEFT_OFF_TXT}")
             )
 
         if b'{"__dr":"CometErrorRoot.react"}' in res.content:
-            raise exception.StopExtraction(
-                "You've been temporarily blocked from viewing images. "
-                "\nPlease try using a different account, "
-                "using a VPN or waiting before you retry." +
-                LEFT_OFF_TXT
+            raise exception.AbortExtraction(
+                f"You've been temporarily blocked from viewing images.\n"
+                f"Please try using a different account, "
+                f"using a VPN or waiting before you retry.{LEFT_OFF_TXT}"
             )
 
         return res
 
-    def extract_set(self, first_photo_id, set_id):
-        all_photo_ids = [first_photo_id]
+    def extract_set(self, set_data):
+        set_id = set_data["set_id"]
+        all_photo_ids = [set_data["first_photo_id"]]
 
         retries = 0
         i = 0
 
         while i < len(all_photo_ids):
             photo_id = all_photo_ids[i]
-            photo_url = self.photo_url_fmt.format(
-                photo_id=photo_id, set_id=set_id
-            )
+            photo_url = f"{self.root}/photo/?fbid={photo_id}&set={set_id}"
             photo_page = self.photo_page_request_wrapper(photo_url).text
 
             photo = self.parse_photo_page(photo_page)
-            photo["set_id"] = set_id
             photo["num"] = i + 1
 
             if self.author_followups:
@@ -280,9 +291,11 @@ class FacebookExtractor(Extractor):
                     retries = 0
             else:
                 retries = 0
+                photo.update(set_data)
+                yield Message.Directory, photo
                 yield Message.Url, photo["url"], photo
 
-            if photo["next_photo_id"] == "":
+            if not photo["next_photo_id"]:
                 self.log.debug(
                     "Can't find next image in the set. "
                     "Extraction is over."
@@ -298,6 +311,70 @@ class FacebookExtractor(Extractor):
 
             i += 1
 
+    @memcache(keyarg=1)
+    def _extract_profile(self, profile, set_id=False):
+        if set_id:
+            url = f"{self.root}/{profile}/photos_by"
+        else:
+            url = f"{self.root}/{profile}"
+        return self._extract_profile_page(url)
+
+    def _extract_profile_page(self, url):
+        for _ in range(self.fallback_retries + 1):
+            page = self.request(url).text
+
+            if page.find('>Page Not Found</title>', 0, 3000) > 0:
+                break
+            if ('"props":{"title":"This content isn\'t available right now"' in
+                    page):
+                raise exception.AuthRequired(
+                    "authenticated cookies", "profile",
+                    "This content isn't available right now")
+
+            set_id = self._extract_profile_set_id(page)
+            user = self._extract_profile_user(page)
+            if set_id or user:
+                user["set_id"] = set_id
+                return user
+
+            self.log.debug("Got empty profile photos page, retrying...")
+        return {}
+
+    def _extract_profile_set_id(self, profile_photos_page):
+        set_ids_raw = text.extr(
+            profile_photos_page, '"pageItems"', '"page_info"'
+        )
+
+        set_id = text.extr(
+            set_ids_raw, 'set=', '"'
+        ).rsplit("&", 1)[0] or text.extr(
+            set_ids_raw, '\\/photos\\/', '\\/'
+        )
+
+        return set_id
+
+    def _extract_profile_user(self, page):
+        data = text.extr(page, '","user":{"', '},"viewer":{')
+
+        user = None
+        try:
+            user = util.json_loads(f'{{"{data}}}')
+            if user["id"].startswith("pfbid"):
+                user["user_pfbid"] = user["id"]
+                user["id"] = text.extr(page, '"userID":"', '"')
+            user["username"] = (text.extr(page, '"userVanity":"', '"') or
+                                text.extr(page, '"vanity":"', '"'))
+            user["profile_tabs"] = [
+                edge["node"]
+                for edge in (user["profile_tabs"]["profile_user"]
+                             ["timeline_nav_app_sections"]["edges"])
+            ]
+        except Exception:
+            if user is None:
+                self.log.debug("Failed to extract user data: %s", data)
+                user = {}
+        return user
+
 
 class FacebookSetExtractor(FacebookExtractor):
     """Base class for Facebook Set extractors"""
@@ -313,23 +390,18 @@ class FacebookSetExtractor(FacebookExtractor):
 
     def items(self):
         set_id = self.groups[0] or self.groups[3]
-        path = self.groups[1]
-        if path:
+        if path := self.groups[1]:
             post_url = self.root + "/" + path
             post_page = self.request(post_url).text
             set_id = self.parse_post_page(post_page)["set_id"]
 
-        set_url = self.set_url_fmt.format(set_id=set_id)
+        set_url = f"{self.root}/media/set/?set={set_id}"
         set_page = self.request(set_url).text
+        set_data = self.parse_set_page(set_page)
+        if self.groups[2]:
+            set_data["first_photo_id"] = self.groups[2]
 
-        directory = self.parse_set_page(set_page)
-
-        yield Message.Directory, directory
-
-        yield from self.extract_set(
-            self.groups[2] or directory["first_photo_id"],
-            directory["set_id"]
-        )
+        return self.extract_set(set_data)
 
 
 class FacebookPhotoExtractor(FacebookExtractor):
@@ -342,16 +414,15 @@ class FacebookPhotoExtractor(FacebookExtractor):
 
     def items(self):
         photo_id = self.groups[0]
-        photo_url = self.photo_url_fmt.format(photo_id=photo_id, set_id="")
+        photo_url = f"{self.root}/photo/?fbid={photo_id}&set="
         photo_page = self.photo_page_request_wrapper(photo_url).text
 
         i = 1
         photo = self.parse_photo_page(photo_page)
         photo["num"] = i
 
-        set_page = self.request(
-            self.set_url_fmt.format(set_id=photo["set_id"])
-        ).text
+        set_url = f"{self.root}/media/set/?set={photo['set_id']}"
+        set_page = self.request(set_url).text
 
         directory = self.parse_set_page(set_page)
 
@@ -362,9 +433,7 @@ class FacebookPhotoExtractor(FacebookExtractor):
             for comment_photo_id in photo["followups_ids"]:
                 comment_photo = self.parse_photo_page(
                     self.photo_page_request_wrapper(
-                        self.photo_url_fmt.format(
-                            photo_id=comment_photo_id, set_id=""
-                        )
+                        f"{self.root}/photo/?fbid={comment_photo_id}&set="
                     ).text
                 )
                 i += 1
@@ -399,49 +468,101 @@ class FacebookVideoExtractor(FacebookExtractor):
                 yield Message.Url, audio["url"], audio
 
 
-class FacebookProfileExtractor(FacebookExtractor):
-    """Base class for Facebook Profile Photos Set extractors"""
-    subcategory = "profile"
-    pattern = (
-        BASE_PATTERN +
-        r"/(?!media/|photo/|photo.php|watch/)"
-        r"(?:profile\.php\?id=|people/[^/?#]+/)?"
-        r"([^/?&#]+)(?:/photos(?:_by)?|/videos|/posts)?/?(?:$|\?|#)"
-    )
-    example = "https://www.facebook.com/USERNAME"
-
-    @staticmethod
-    def get_profile_photos_set_id(profile_photos_page):
-        set_ids_raw = text.extr(
-            profile_photos_page, '"pageItems"', '"page_info"'
-        )
-
-        set_id = text.extr(
-            set_ids_raw, 'set=', '"'
-        ).rsplit("&", 1)[0] or text.extr(
-            set_ids_raw, '\\/photos\\/', '\\/'
-        )
-
-        return set_id
+class FacebookInfoExtractor(FacebookExtractor):
+    """Extractor for Facebook Profile data"""
+    subcategory = "info"
+    directory_fmt = ("{category}", "{username}")
+    pattern = USER_PATTERN + r"/info"
+    example = "https://www.facebook.com/USERNAME/info"
 
     def items(self):
-        profile_photos_url = (
-            self.root + "/" + self.groups[0] + "/photos_by"
-        )
-        profile_photos_page = self.request(profile_photos_url).text
+        user = self._extract_profile(self.groups[0])
+        return iter(((Message.Directory, user),))
 
-        set_id = self.get_profile_photos_set_id(profile_photos_page)
 
-        if set_id:
-            set_url = self.set_url_fmt.format(set_id=set_id)
-            set_page = self.request(set_url).text
+class FacebookAlbumsExtractor(FacebookExtractor):
+    """Extractor for Facebook Profile albums"""
+    subcategory = "albums"
+    pattern = USER_PATTERN + r"/photos_albums(?:/([^/?#]+))?"
+    example = "https://www.facebook.com/USERNAME/photos_albums"
 
-            directory = self.parse_set_page(set_page)
+    def items(self):
+        profile, name = self.groups
+        url = f"{self.root}/{profile}/photos_albums"
+        page = self.request(url).text
 
-            yield Message.Directory, directory
+        pos = page.find(
+            '"TimelineAppCollectionAlbumsRenderer","collection":{"id":"')
+        if pos < 0:
+            return
+        if name is not None:
+            name = name.lower()
 
-            yield from self.extract_set(
-                directory["first_photo_id"], directory["set_id"]
-            )
-        else:
-            self.log.debug("Profile photos set ID not found.")
+        items = text.extract(page, '},"pageItems":', '}}},', pos)[0]
+        edges = util.json_loads(items + "}}")["edges"]
+
+        # TODO: use /graphql API endpoint
+        for edge in edges:
+            node = edge["node"]
+            album = node["node"]
+            album["title"] = title = node["title"]["text"]
+            if name is not None and name != title.lower():
+                continue
+            album["_extractor"] = FacebookSetExtractor
+            album["thumbnail"] = (img := node["image"]) and img["uri"]
+            yield Message.Queue, album["url"], album
+
+
+class FacebookPhotosExtractor(FacebookExtractor):
+    """Extractor for Facebook Profile Photos"""
+    subcategory = "photos"
+    pattern = USER_PATTERN + r"/photos(?:_by)?"
+    example = "https://www.facebook.com/USERNAME/photos"
+
+    def items(self):
+        set_id = self._extract_profile(self.groups[0], True)["set_id"]
+        if not set_id:
+            return iter(())
+
+        set_url = f"{self.root}/media/set/?set={set_id}"
+        set_page = self.request(set_url).text
+        set_data = self.parse_set_page(set_page)
+        return self.extract_set(set_data)
+
+
+class FacebookAvatarExtractor(FacebookExtractor):
+    """Extractor for Facebook Profile Avatars"""
+    subcategory = "avatar"
+    pattern = USER_PATTERN + r"/avatar"
+    example = "https://www.facebook.com/USERNAME/avatar"
+
+    def items(self):
+        user = self._extract_profile(self.groups[0])
+        avatar_page_url = user["profilePhoto"]["url"]
+        avatar_page = self.photo_page_request_wrapper(avatar_page_url).text
+
+        avatar = self.parse_photo_page(avatar_page)
+        avatar["count"] = avatar["num"] = 1
+        avatar["type"] = "avatar"
+
+        set_url = f"{self.root}/media/set/?set={avatar['set_id']}"
+        set_page = self.request(set_url).text
+        directory = self.parse_set_page(set_page)
+
+        yield Message.Directory, directory
+        yield Message.Url, avatar["url"], avatar
+
+
+class FacebookUserExtractor(Dispatch, FacebookExtractor):
+    """Extractor for Facebook Profiles"""
+    pattern = USER_PATTERN + r"/?(?:$|\?|#)"
+    example = "https://www.facebook.com/USERNAME"
+
+    def items(self):
+        base = f"{self.root}/{self.groups[0]}/"
+        return self._dispatch_extractors((
+            (FacebookInfoExtractor  , base + "info"),
+            (FacebookAvatarExtractor, base + "avatar"),
+            (FacebookPhotosExtractor, base + "photos"),
+            (FacebookAlbumsExtractor, base + "photos_albums"),
+        ), ("photos",))
