@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 # Copyright 2018-2020 Leonardo Taccari
-# Copyright 2018-2023 Mike Fährmann
+# Copyright 2018-2025 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -9,12 +9,11 @@
 
 """Extractors for https://www.instagram.com/"""
 
-from .common import Extractor, Message
+from .common import Extractor, Message, Dispatch
 from .. import text, util, exception
 from ..cache import cache, memcache
 import itertools
 import binascii
-import re
 
 BASE_PATTERN = r"(?:https?://)?(?:www\.)?instagram\.com"
 USER_PATTERN = BASE_PATTERN + r"/(?!(?:p|tv|reel|explore|stories)/)([^/?#]+)"
@@ -34,12 +33,13 @@ class InstagramExtractor(Extractor):
 
     def __init__(self, match):
         Extractor.__init__(self, match)
-        self.item = match.group(1)
+        self.item = match[1]
 
     def _init(self):
         self.www_claim = "0"
         self.csrf_token = util.generate_token()
-        self._find_tags = re.compile(r"#\w+").findall
+        self._find_tags = util.re(r"#\w+").findall
+        self._warn_video_ua = True
         self._logged_in = True
         self._cursor = None
         self._user = None
@@ -56,8 +56,7 @@ class InstagramExtractor(Extractor):
         self.login()
 
         data = self.metadata()
-        videos = self.config("videos", True)
-        if videos:
+        if videos := self.config("videos", True):
             videos_dash = (videos != "merged")
             videos_headers = {"User-Agent": "Mozilla/5.0"}
         previews = self.config("previews", False)
@@ -92,15 +91,14 @@ class InstagramExtractor(Extractor):
             for file in files:
                 file.update(post)
 
-                url = file.get("video_url")
-                if url:
+                if url := file.get("video_url"):
                     if videos:
                         file["_http_headers"] = videos_headers
                         text.nameext_from_url(url, file)
                         if videos_dash:
                             file["_fallback"] = (url,)
                             file["_ytdl_manifest"] = "dash"
-                            url = "ytdl:dash"
+                            url = f"ytdl:{post['post_url']}{file['num']}.mp4"
                         yield Message.Url, url, file
                     if previews:
                         file["media_id"] += "p"
@@ -137,16 +135,15 @@ class InstagramExtractor(Extractor):
             else:
                 page = None
 
-            if page:
-                raise exception.StopExtraction("HTTP redirect to %s page (%s)",
-                                               page, url.partition("?")[0])
+            if page is not None:
+                raise exception.AbortExtraction(
+                    f"HTTP redirect to {page} page ({url.partition('?')[0]})")
 
         www_claim = response.headers.get("x-ig-set-www-claim")
         if www_claim is not None:
             self.www_claim = www_claim
 
-        csrf_token = response.cookies.get("csrftoken")
-        if csrf_token:
+        if csrf_token := response.cookies.get("csrftoken"):
             self.csrf_token = csrf_token
 
         return response
@@ -165,19 +162,27 @@ class InstagramExtractor(Extractor):
         if "items" in post:  # story or highlight
             items = post["items"]
             reel_id = str(post["id"]).rpartition(":")[2]
+            if expires := post.get("expiring_at"):
+                post_url = f"{self.root}/stories/{post['user']['username']}/"
+            else:
+                post_url = f"{self.root}/stories/highlights/{reel_id}/"
             data = {
-                "expires": text.parse_timestamp(post.get("expiring_at")),
+                "user"   : post.get("user"),
+                "expires": text.parse_timestamp(expires),
                 "post_id": reel_id,
                 "post_shortcode": shortcode_from_id(reel_id),
+                "post_url": post_url,
             }
             if "title" in post:
                 data["highlight_title"] = post["title"]
+            if expires and not post.get("seen"):
+                post["seen"] = expires - 86400
 
         else:  # regular image/video post
             data = {
                 "post_id" : post["pk"],
                 "post_shortcode": post["code"],
-                "post_url": "{}/p/{}/".format(self.root, post["code"]),
+                "post_url": f"{self.root}/p/{post['code']}/",
                 "likes": post.get("like_count", 0),
                 "liked": post.get("has_liked", False),
                 "pinned": self._extract_pinned(post),
@@ -186,20 +191,17 @@ class InstagramExtractor(Extractor):
             caption = post["caption"]
             data["description"] = caption["text"] if caption else ""
 
-            tags = self._find_tags(data["description"])
-            if tags:
+            if tags := self._find_tags(data["description"]):
                 data["tags"] = sorted(set(tags))
 
-            location = post.get("location")
-            if location:
+            if location := post.get("location"):
                 slug = location["short_name"].replace(" ", "-").lower()
                 data["location_id"] = location["pk"]
                 data["location_slug"] = slug
-                data["location_url"] = "{}/explore/locations/{}/{}/".format(
-                    self.root, location["pk"], slug)
+                data["location_url"] = \
+                    f"{self.root}/explore/locations/{location['pk']}/{slug}/"
 
-            coauthors = post.get("coauthor_producers")
-            if coauthors:
+            if coauthors := post.get("coauthor_producers"):
                 data["coauthors"] = [
                     {"id"       : user["pk"],
                      "username" : user["username"],
@@ -207,8 +209,7 @@ class InstagramExtractor(Extractor):
                     for user in coauthors
                 ]
 
-            items = post.get("carousel_media")
-            if items:
+            if items := post.get("carousel_media"):
                 data["sidecar_media_id"] = data["post_id"]
                 data["sidecar_shortcode"] = data["post_shortcode"]
             else:
@@ -230,17 +231,37 @@ class InstagramExtractor(Extractor):
                                  data["post_shortcode"])
                 continue
 
-            video_versions = item.get("video_versions")
-            if video_versions:
+            if video_versions := item.get("video_versions"):
                 video = max(
                     video_versions,
                     key=lambda x: (x["width"], x["height"], x["type"]),
                 )
+                manifest = item.get("video_dash_manifest")
                 media = video
+
+                if self._warn_video_ua:
+                    self._warn_video_ua = False
+                    pattern = text.re(
+                        r"AppleWebKit/537\.36 \(KHTML, like Gecko\) "
+                        r"Chrome/\d+\.\d+\.\d+\.\d+ Safari/537\.36$")
+                    if not pattern.search(self.session.headers["User-Agent"]):
+                        self.log.warning("Potentially lowered video quality "
+                                         "due to non-Chrome User-Agent")
             else:
-                video = None
+                video = manifest = None
                 media = image
 
+                if image["width"] < item.get("original_width", 0) or \
+                        image["height"] < item.get("original_height", 0):
+                    self.log.warning(
+                        "%s: Available image resolutions lower than the "
+                        "original (%sx%s < %sx%s). "
+                        "Consider refreshing your cookies.",
+                        data["post_shortcode"],
+                        image["width"], image["height"],
+                        item.get("original_width", 0),
+                        item.get("original_height", 0))
+
             media = {
                 "num"        : num,
                 "date"       : text.parse_timestamp(item.get("taken_at") or
@@ -253,9 +274,14 @@ class InstagramExtractor(Extractor):
                 "video_url"  : video["url"] if video else None,
                 "width"      : media["width"],
                 "height"     : media["height"],
-                "_ytdl_manifest_data": item.get("video_dash_manifest"),
             }
 
+            if manifest is not None:
+                media["_ytdl_manifest_data"] = manifest
+            if "owner" in item:
+                media["owner2"] = item["owner"]
+            if "reshared_story_media_author" in item:
+                media["author"] = item["reshared_story_media_author"]
             if "expiring_at" in item:
                 media["expires"] = text.parse_timestamp(post["expiring_at"])
 
@@ -274,8 +300,7 @@ class InstagramExtractor(Extractor):
                     "edge_sidecar_to_children" not in post:
                 post = self.api.media(post["id"])[0]
 
-        pinned = post.get("pinned_for_users", ())
-        if pinned:
+        if pinned := post.get("pinned_for_users", ()):
             for index, user in enumerate(pinned):
                 pinned[index] = int(user["id"])
 
@@ -290,7 +315,7 @@ class InstagramExtractor(Extractor):
             "fullname"   : owner.get("full_name"),
             "post_id"    : post["id"],
             "post_shortcode": post["shortcode"],
-            "post_url"   : "{}/p/{}/".format(self.root, post["shortcode"]),
+            "post_url"   : f"{self.root}/p/{post['shortcode']}/",
             "post_date"  : text.parse_timestamp(post["taken_at_timestamp"]),
             "description": text.parse_unicode_escapes("\n".join(
                 edge["node"]["text"]
@@ -299,19 +324,16 @@ class InstagramExtractor(Extractor):
         }
         data["date"] = data["post_date"]
 
-        tags = self._find_tags(data["description"])
-        if tags:
+        if tags := self._find_tags(data["description"]):
             data["tags"] = sorted(set(tags))
 
-        location = post.get("location")
-        if location:
+        if location := post.get("location"):
             data["location_id"] = location["id"]
             data["location_slug"] = location["slug"]
-            data["location_url"] = "{}/explore/locations/{}/{}/".format(
-                self.root, location["id"], location["slug"])
+            data["location_url"] = (f"{self.root}/explore/locations/"
+                                    f"{location['id']}/{location['slug']}/")
 
-        coauthors = post.get("coauthor_producers")
-        if coauthors:
+        if coauthors := post.get("coauthor_producers"):
             data["coauthors"] = [
                 {"id"      : user["id"],
                  "username": user["username"]}
@@ -355,36 +377,31 @@ class InstagramExtractor(Extractor):
 
         return data
 
-    @staticmethod
-    def _extract_tagged_users(src, dest):
+    def _extract_tagged_users(self, src, dest):
         dest["tagged_users"] = tagged_users = []
 
-        edges = src.get("edge_media_to_tagged_user")
-        if edges:
+        if edges := src.get("edge_media_to_tagged_user"):
             for edge in edges["edges"]:
                 user = edge["node"]["user"]
                 tagged_users.append({"id"       : user["id"],
                                      "username" : user["username"],
                                      "full_name": user["full_name"]})
 
-        usertags = src.get("usertags")
-        if usertags:
+        if usertags := src.get("usertags"):
             for tag in usertags["in"]:
                 user = tag["user"]
                 tagged_users.append({"id"       : user["pk"],
                                      "username" : user["username"],
                                      "full_name": user["full_name"]})
 
-        mentions = src.get("reel_mentions")
-        if mentions:
+        if mentions := src.get("reel_mentions"):
             for mention in mentions:
                 user = mention["user"]
                 tagged_users.append({"id"       : user.get("pk"),
                                      "username" : user["username"],
                                      "full_name": user["full_name"]})
 
-        stickers = src.get("story_bloks_stickers")
-        if stickers:
+        if stickers := src.get("story_bloks_stickers"):
             for sticker in stickers:
                 sticker = sticker["bloks_sticker"]
                 if sticker["bloks_sticker_type"] == "mention":
@@ -427,21 +444,14 @@ class InstagramExtractor(Extractor):
                 user[key] = 0
 
 
-class InstagramUserExtractor(InstagramExtractor):
+class InstagramUserExtractor(Dispatch, InstagramExtractor):
     """Extractor for an Instagram user profile"""
-    subcategory = "user"
     pattern = USER_PATTERN + r"/?(?:$|[?#])"
     example = "https://www.instagram.com/USER/"
 
-    def initialize(self):
-        pass
-
-    def finalize(self):
-        pass
-
     def items(self):
-        base = "{}/{}/".format(self.root, self.item)
-        stories = "{}/stories/{}/".format(self.root, self.item)
+        base = f"{self.root}/{self.item}/"
+        stories = f"{self.root}/stories/{self.item}/"
         return self._dispatch_extractors((
             (InstagramInfoExtractor      , base + "info/"),
             (InstagramAvatarExtractor    , base + "avatar/"),
@@ -519,7 +529,7 @@ class InstagramGuideExtractor(InstagramExtractor):
 
     def __init__(self, match):
         InstagramExtractor.__init__(self, match)
-        self.guide_id = match.group(2)
+        self.guide_id = match[2]
 
     def metadata(self):
         return {"guide": self.api.guide(self.guide_id)}
@@ -583,7 +593,10 @@ class InstagramStoriesExtractor(InstagramExtractor):
         reel_id = self.highlight_id or self.api.user_id(self.user)
         reels = self.api.reels_media(reel_id)
 
-        if self.media_id and reels:
+        if not reels:
+            return ()
+
+        if self.media_id:
             reel = reels[0]
             for item in reel["items"]:
                 if item["pk"] == self.media_id:
@@ -592,6 +605,16 @@ class InstagramStoriesExtractor(InstagramExtractor):
             else:
                 raise exception.NotFoundError("story")
 
+        elif self.config("split"):
+            reel = reels[0]
+            reels = []
+            for item in reel["items"]:
+                item.pop("user", None)
+                copy = reel.copy()
+                copy.update(item)
+                copy["items"] = (item,)
+                reels.append(copy)
+
         return reels
 
 
@@ -616,7 +639,7 @@ class InstagramFollowersExtractor(InstagramExtractor):
         uid = self.api.user_id(self.item)
         for user in self.api.user_followers(uid):
             user["_extractor"] = InstagramUserExtractor
-            url = "{}/{}".format(self.root, user["username"])
+            url = f"{self.root}/{user['username']}"
             yield Message.Queue, url, user
 
 
@@ -630,7 +653,7 @@ class InstagramFollowingExtractor(InstagramExtractor):
         uid = self.api.user_id(self.item)
         for user in self.api.user_following(uid):
             user["_extractor"] = InstagramUserExtractor
-            url = "{}/{}".format(self.root, user["username"])
+            url = f"{self.root}/{user['username']}"
             yield Message.Queue, url, user
 
 
@@ -686,8 +709,7 @@ class InstagramAvatarExtractor(InstagramExtractor):
             url = user.get("profile_pic_url_hd") or user["profile_pic_url"]
             avatar = {"url": url, "width": 0, "height": 0}
 
-        pk = user.get("profile_pic_id")
-        if pk:
+        if pk := user.get("profile_pic_id"):
             pk = pk.partition("_")[0]
             code = shortcode_from_id(pk)
         else:
@@ -735,14 +757,13 @@ class InstagramRestAPI():
         return self._call(endpoint, params=params)
 
     def guide_media(self, guide_id):
-        endpoint = "/v1/guides/guide/{}/".format(guide_id)
+        endpoint = f"/v1/guides/guide/{guide_id}/"
         return self._pagination_guides(endpoint)
 
     def highlights_media(self, user_id, chunk_size=5):
         reel_ids = [hl["id"] for hl in self.highlights_tray(user_id)]
 
-        order = self.extractor.config("order-posts")
-        if order:
+        if order := self.extractor.config("order-posts"):
             if order in ("desc", "reverse"):
                 reel_ids.reverse()
             elif order in ("id", "id_asc"):
@@ -757,13 +778,13 @@ class InstagramRestAPI():
                 reel_ids[offset : offset+chunk_size])
 
     def highlights_tray(self, user_id):
-        endpoint = "/v1/highlights/{}/highlights_tray/".format(user_id)
+        endpoint = f"/v1/highlights/{user_id}/highlights_tray/"
         return self._call(endpoint)["tray"]
 
     def media(self, shortcode):
         if len(shortcode) > 28:
             shortcode = shortcode[:-28]
-        endpoint = "/v1/media/{}/info/".format(id_from_shortcode(shortcode))
+        endpoint = f"/v1/media/{id_from_shortcode(shortcode)}/info/"
         return self._pagination(endpoint)
 
     def reels_media(self, reel_ids):
@@ -780,7 +801,7 @@ class InstagramRestAPI():
                 yield media["media"]
 
     def tags_sections(self, tag):
-        endpoint = "/v1/tags/{}/sections/".format(tag)
+        endpoint = f"/v1/tags/{tag}/sections/"
         data = {
             "include_persistent": "0",
             "max_id" : None,
@@ -799,7 +820,7 @@ class InstagramRestAPI():
 
     @memcache(keyarg=1)
     def user_by_id(self, user_id):
-        endpoint = "/v1/users/{}/info/".format(user_id)
+        endpoint = f"/v1/users/{user_id}/info/"
         return self._call(endpoint)["user"]
 
     def user_id(self, screen_name, check_private=True):
@@ -831,22 +852,22 @@ class InstagramRestAPI():
         return self._pagination_post(endpoint, data)
 
     def user_collection(self, collection_id):
-        endpoint = "/v1/feed/collection/{}/posts/".format(collection_id)
+        endpoint = f"/v1/feed/collection/{collection_id}/posts/"
         params = {"count": 50}
         return self._pagination(endpoint, params, media=True)
 
     def user_feed(self, user_id):
-        endpoint = "/v1/feed/user/{}/".format(user_id)
+        endpoint = f"/v1/feed/user/{user_id}/"
         params = {"count": 30}
         return self._pagination(endpoint, params)
 
     def user_followers(self, user_id):
-        endpoint = "/v1/friendships/{}/followers/".format(user_id)
+        endpoint = f"/v1/friendships/{user_id}/followers/"
         params = {"count": 12}
         return self._pagination_following(endpoint, params)
 
     def user_following(self, user_id):
-        endpoint = "/v1/friendships/{}/following/".format(user_id)
+        endpoint = f"/v1/friendships/{user_id}/following/"
         params = {"count": 12}
         return self._pagination_following(endpoint, params)
 
@@ -856,7 +877,7 @@ class InstagramRestAPI():
         return self._pagination(endpoint, params, media=True)
 
     def user_tagged(self, user_id):
-        endpoint = "/v1/usertags/{}/feed/".format(user_id)
+        endpoint = f"/v1/usertags/{user_id}/feed/"
         params = {"count": 20}
         return self._pagination(endpoint, params)
 
@@ -877,7 +898,7 @@ class InstagramRestAPI():
             "Sec-Fetch-Mode"  : "cors",
             "Sec-Fetch-Site"  : "same-origin",
         }
-        return extr.request(url, **kwargs).json()
+        return extr.request_json(url, **kwargs)
 
     def _pagination(self, endpoint, params=None, media=False):
         if params is None:
@@ -971,9 +992,8 @@ class InstagramGraphqlAPI():
         self.user_by_id = api.user_by_id
         self.user_id = api.user_id
 
-    @staticmethod
-    def _unsupported(_=None):
-        raise exception.StopExtraction("Unsupported with GraphQL API")
+    def _unsupported(self, _=None):
+        raise exception.AbortExtraction("Unsupported with GraphQL API")
 
     def highlights_tray(self, user_id):
         query_hash = "d4d88dc1500312af6f937f7b804c68c3"
@@ -1041,7 +1061,7 @@ class InstagramGraphqlAPI():
             "X-Requested-With": "XMLHttpRequest",
             "Referer"         : extr.root + "/",
         }
-        return extr.request(url, params=params, headers=headers).json()["data"]
+        return extr.request_json(url, params=params, headers=headers)["data"]
 
     def _pagination(self, query_hash, variables,
                     key_data="user", key_edge=None):
@@ -1059,9 +1079,10 @@ class InstagramGraphqlAPI():
             if not info["has_next_page"]:
                 return extr._update_cursor(None)
             elif not data["edges"]:
-                s = "" if self.extractor.item.endswith("s") else "s"
-                raise exception.StopExtraction(
-                    "%s'%s posts are private", self.extractor.item, s)
+                user = self.extractor.item
+                s = "" if user.endswith("s") else "s"
+                raise exception.AbortExtraction(
+                    f"{user}'{s} posts are private")
 
             variables["after"] = extr._update_cursor(info["end_cursor"])
 
