@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
-# Copyright 2017-2023 Mike Fährmann
+# Copyright 2017-2025 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -14,6 +14,7 @@ from unittest import mock
 
 import time
 import logging
+import datetime
 import tempfile
 from os.path import join
 
@@ -70,8 +71,7 @@ class TestCookiejar(unittest.TestCase):
 
         self.assertEqual(len(cookies), 0)
         self.assertEqual(mock_warning.call_count, 1)
-        self.assertEqual(mock_warning.call_args[0][0], "cookies: %s")
-        self.assertIsInstance(mock_warning.call_args[0][1], exc)
+        self.assertIsInstance(mock_warning.call_args[0][-1], exc)
 
 
 class TestCookiedict(unittest.TestCase):
@@ -91,7 +91,7 @@ class TestCookiedict(unittest.TestCase):
         self.assertEqual(sorted(cookies.values()), sorted(self.cdict.values()))
 
     def test_domain(self):
-        for category in ["exhentai", "idolcomplex", "nijie", "horne"]:
+        for category in ["exhentai", "nijie", "horne"]:
             extr = _get_extractor(category)
             cookies = extr.cookies
             for key in self.cdict:
@@ -108,7 +108,6 @@ class TestCookieLogin(unittest.TestCase):
     def test_cookie_login(self):
         extr_cookies = {
             "exhentai"   : ("ipb_member_id", "ipb_pass_hash"),
-            "idolcomplex": ("login", "pass_hash"),
             "nijie"      : ("nijie_tok",),
             "horne"      : ("horne_tok",),
         }
@@ -159,7 +158,7 @@ class TestCookieUtils(unittest.TestCase):
         extr.cookies.set("cd_a", "1", domain=extr.cookies_domain)
         self.assertTrue(extr.cookies_check(("cd_a",)))
 
-        extr.cookies.set("wd_a", "1", domain="www" + extr.cookies_domain)
+        extr.cookies.set("wd_a", "1", domain=f"www{extr.cookies_domain}")
         self.assertFalse(extr.cookies_check(("wd_a",)))
         self.assertEqual(len(extr.cookies), 3)
 
@@ -184,7 +183,7 @@ class TestCookieUtils(unittest.TestCase):
         extr.cookies.set("cd_a", "1", domain=extr.cookies_domain)
         self.assertTrue(extr.cookies_check(("cd_a",), subdomains=True))
 
-        extr.cookies.set("wd_a", "1", domain="www" + extr.cookies_domain)
+        extr.cookies.set("wd_a", "1", domain=f"www{extr.cookies_domain}")
         self.assertTrue(extr.cookies_check(("wd_a",), subdomains=True))
 
         extr.cookies.set("cd_b", "2", domain=extr.cookies_domain)
@@ -205,27 +204,32 @@ class TestCookieUtils(unittest.TestCase):
         now = int(time.time())
         log = logging.getLogger("generic")
 
-        extr.cookies.set("a", "1", expires=now-100)
+        extr.cookies.set("a", "1", expires=now-100, domain=".example.org")
         with mock.patch.object(log, "warning") as mw:
             self.assertFalse(extr.cookies_check(("a",)))
             self.assertEqual(mw.call_count, 1)
-            self.assertEqual(mw.call_args[0], ("Cookie '%s' has expired", "a"))
+            self.assertEqual(mw.call_args[0], (
+                "cookies: %s/%s expired at %s", "example.org", "a",
+                datetime.datetime.fromtimestamp(now-100)))
 
-        extr.cookies.set("a", "1", expires=now+100)
+        extr.cookies.set("a", "1", expires=now+100, domain=".example.org")
         with mock.patch.object(log, "warning") as mw:
             self.assertTrue(extr.cookies_check(("a",)))
             self.assertEqual(mw.call_count, 1)
             self.assertEqual(mw.call_args[0], (
-                "Cookie '%s' will expire in less than %s hour%s", "a", 1, ""))
+                "cookies: %s/%s will expire in less than %s hour%s",
+                "example.org", "a", 1, ""))
 
-        extr.cookies.set("a", "1", expires=now+100+7200)
+        extr.cookies.set("a", "1", expires=now+100+7200, domain=".example.org")
         with mock.patch.object(log, "warning") as mw:
             self.assertTrue(extr.cookies_check(("a",)))
             self.assertEqual(mw.call_count, 1)
             self.assertEqual(mw.call_args[0], (
-                "Cookie '%s' will expire in less than %s hour%s", "a", 3, "s"))
+                "cookies: %s/%s will expire in less than %s hour%s",
+                "example.org", "a", 3, "s"))
 
-        extr.cookies.set("a", "1", expires=now+100+24*3600)
+        extr.cookies.set(
+            "a", "1", expires=now+100+24*3600, domain=".example.org")
         with mock.patch.object(log, "warning") as mw:
             self.assertTrue(extr.cookies_check(("a",)))
             self.assertEqual(mw.call_count, 0)
@@ -239,7 +243,6 @@ def _get_extractor(category):
 
 URLS = {
     "exhentai"   : "https://exhentai.org/g/1200119/d55c44d3d0/",
-    "idolcomplex": "https://idol.sankakucomplex.com/post/show/1",
     "nijie"      : "https://nijie.info/view.php?id=1",
     "horne"      : "https://horne.red/view.php?id=1",
     "test"       : "generic:https://example.org/",
