@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
-# Copyright 2021 Mike Fährmann
+# Copyright 2021-2025 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -13,7 +13,6 @@ import unittest
 from unittest.mock import patch
 
 import io
-import contextlib
 
 sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
 from gallery_dl import job, config, text  # noqa E402
@@ -32,8 +31,13 @@ class TestJob(unittest.TestCase):
             jobinstance = extr_or_job
 
         with io.StringIO() as buffer:
-            with contextlib.redirect_stdout(buffer):
+            stdout = sys.stdout
+            sys.stdout = buffer
+            try:
                 jobinstance.run()
+            finally:
+                sys.stdout = stdout
+
             return buffer.getvalue()
 
 
@@ -66,17 +70,36 @@ class TestKeywordJob(TestJob):
     jobclass = job.KeywordJob
 
     def test_default(self):
-        extr = TestExtractor.from_url("test:")
+        self.maxDiff = None
+        extr = TestExtractor.from_url("test:self")
         self.assertEqual(self._capture_stdout(extr), """\
 Keywords for directory names:
 -----------------------------
+author['id']
+  123
+author['name']
+  test
+author['self']
+  <circular reference>
 category
   test_category
 subcategory
   test_subcategory
+user['id']
+  123
+user['name']
+  test
+user['self']
+  <circular reference>
 
 Keywords for filenames and --filter:
 ------------------------------------
+author['id']
+  123
+author['name']
+  test
+author['self']
+  <circular reference>
 category
   test_category
 extension
@@ -91,10 +114,12 @@ tags[N]
   0 foo
   1 bar
   2 テスト
-user[id]
+user['id']
   123
-user[name]
+user['name']
   test
+user['self']
+  <circular reference>
 """)
 
 
@@ -209,6 +234,7 @@ class TestDataJob(TestJob):
     def test_default(self):
         extr = TestExtractor.from_url("test:")
         tjob = self.jobclass(extr, file=io.StringIO())
+        user = {"id": 123, "name": "test"}
 
         tjob.run()
 
@@ -216,6 +242,8 @@ class TestDataJob(TestJob):
             (Message.Directory, {
                 "category"   : "test_category",
                 "subcategory": "test_subcategory",
+                "user"       : user,
+                "author"     : user,
             }),
             (Message.Url, "https://example.org/1.jpg", {
                 "category"   : "test_category",
@@ -224,7 +252,8 @@ class TestDataJob(TestJob):
                 "extension"  : "jpg",
                 "num"        : 1,
                 "tags"       : ["foo", "bar", "テスト"],
-                "user"       : {"id": 123, "name": "test"},
+                "user"       : user,
+                "author"     : user,
             }),
             (Message.Url, "https://example.org/2.jpg", {
                 "category"   : "test_category",
@@ -233,7 +262,8 @@ class TestDataJob(TestJob):
                 "extension"  : "jpg",
                 "num"        : 2,
                 "tags"       : ["foo", "bar", "テスト"],
-                "user"       : {"id": 123, "name": "test"},
+                "user"       : user,
+                "author"     : user,
             }),
             (Message.Url, "https://example.org/3.jpg", {
                 "category"   : "test_category",
@@ -242,7 +272,8 @@ class TestDataJob(TestJob):
                 "extension"  : "jpg",
                 "num"        : 3,
                 "tags"       : ["foo", "bar", "テスト"],
-                "user"       : {"id": 123, "name": "test"},
+                "user"       : user,
+                "author"     : user,
             }),
         ])
 
@@ -251,7 +282,12 @@ class TestDataJob(TestJob):
         tjob = self.jobclass(extr, file=io.StringIO())
         tjob.run()
         self.assertEqual(
-            tjob.data[-1], ("ZeroDivisionError", "division by zero"))
+            tjob.data[-1],
+            (-1, {
+                "error"  : "ZeroDivisionError",
+                "message": "division by zero",
+            })
+        )
 
     def test_private(self):
         config.set(("output",), "private", True)
@@ -263,7 +299,7 @@ class TestDataJob(TestJob):
         for i in range(1, 4):
             self.assertEqual(
                 tjob.data[i][2]["_fallback"],
-                ("https://example.org/alt/{}.jpg".format(i),),
+                (f"https://example.org/alt/{i}.jpg",),
             )
 
     def test_sleep(self):
@@ -316,7 +352,7 @@ class TestDataJob(TestJob):
         config.set(("output",), "num-to-str", True)
         with patch("gallery_dl.util.number_to_string") as nts:
             tjob.run()
-        self.assertEqual(len(nts.call_args_list), 52)
+        self.assertEqual(len(nts.call_args_list), 72)
 
         tjob.run()
         self.assertEqual(tjob.data[-1][0], Message.Url)
@@ -328,19 +364,31 @@ class TestExtractor(Extractor):
     subcategory = "test_subcategory"
     directory_fmt = ("{category}",)
     filename_fmt = "test_{filename}.{extension}"
-    pattern = r"test:(child)?$"
+    pattern = r"test:(child|self)?$"
+
+    def __init__(self, match):
+        Extractor.__init__(self, match)
+        self.user = {"id": 123, "name": "test"}
+        if match[1] == "self":
+            self.user["self"] = self.user
 
     def items(self):
         root = "https://example.org"
+        user = self.user
+
+        yield Message.Directory, {
+            "user": user,
+            "author": user,
+        }
 
-        yield Message.Directory, {}
         for i in range(1, 4):
-            url = "{}/{}.jpg".format(root, i)
+            url = f"{root}/{i}.jpg"
             yield Message.Url, url, text.nameext_from_url(url, {
                 "num" : i,
                 "tags": ["foo", "bar", "テスト"],
-                "user": {"id": 123, "name": "test"},
-                "_fallback": ("{}/alt/{}.jpg".format(root, i),),
+                "user": user,
+                "author": user,
+                "_fallback": (f"{root}/alt/{i}.jpg",),
             })
 
 
@@ -374,5 +422,5 @@ class TestExtractorAlt(Extractor):
     subcategory = "test_subcategory"
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     unittest.main()
