@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
-# Copyright 2022-2023 Mike Fährmann
+# Copyright 2022-2025 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -22,10 +22,11 @@ class Test_CommandlineArguments(unittest.TestCase):
     def setUpClass(cls):
         try:
             cls.module = __import__(cls.module_name)
-        except ImportError:
-            raise unittest.SkipTest("cannot import module '{}'".format(
-                cls.module_name))
+        except (ImportError, SyntaxError):
+            raise unittest.SkipTest(
+                f"cannot import module '{cls.module_name}'")
         cls.default = ytdl.parse_command_line(cls.module, [])
+        cls.ytdlp = hasattr(cls.module, "cookies")
 
     def test_ignore_errors(self):
         self._("--ignore-errors" , "ignoreerrors", True)
@@ -155,21 +156,21 @@ class Test_CommandlineArguments(unittest.TestCase):
     def test_subs(self):
         opts = self._(["--convert-subs", "srt"])
         conv = {"key": "FFmpegSubtitlesConvertor", "format": "srt"}
-        if self.module_name == "yt_dlp":
+        if self.ytdlp:
             conv["when"] = "before_dl"
         self.assertEqual(opts["postprocessors"][0], conv)
 
     def test_embed(self):
         subs = {"key": "FFmpegEmbedSubtitle"}
         thumb = {"key": "EmbedThumbnail", "already_have_thumbnail": False}
-        if self.module_name == "yt_dlp":
+        if self.ytdlp:
             subs["already_have_subtitle"] = False
 
         opts = self._(["--embed-subs", "--embed-thumbnail"])
         self.assertEqual(opts["postprocessors"][:2], [subs, thumb])
 
         thumb["already_have_thumbnail"] = True
-        if self.module_name == "yt_dlp":
+        if self.ytdlp:
             subs["already_have_subtitle"] = True
             thumb["already_have_thumbnail"] = "all"
 
@@ -212,7 +213,7 @@ class Test_CommandlineArguments(unittest.TestCase):
             "--ignore-config",
         ]
 
-        if self.module_name != "yt_dlp":
+        if not self.ytdlp:
             cmdline.extend((
                 "--dump-json",
                 "--dump-single-json",
